from django.db import models
from brpatientApp.models import Patient
from clmainApp.models import User,HourSlot,Branch
# Create your models here.
#1NW New / جديد
#2AT Attend / تم الحضور
#3AB1 Absence1 / غياب 1 
#4AB2 Absence1 / غياب 2 
#5AB3 Absence1 / غياب 3 
#6CL Cancel / إلغاء
class SessionStatus(models.Model):
    name = models.CharField(max_length=50)
    code = models.CharField(max_length=10,primary_key=True)
    is_paid = models.BooleanField(default=False)
    is_absence =  models.BooleanField(default=False)
    absence_order =  models.IntegerField(default=0)
    color_code = models.CharField(max_length=50, null=True, blank=True)
    def __str__(self):
        return self.name
    
class Session(models.Model):
    branch = models.ForeignKey(Branch,on_delete=models.PROTECT)   
    name = models.CharField(max_length=250)
    price = models.IntegerField(default=0)
    package_price = models.IntegerField(default=0)
    package_min_sessions = models.IntegerField(default=0)
    is_active = models.BooleanField(default=True)
    def __str__(self):
        return self.name

class PatientSession(models.Model):
    patient =  models.ForeignKey(
        Patient,
        related_name="PatientSession_patient",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    session = models.ForeignKey(
        Session,
        related_name="PatientSession_session",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    session_price = models.IntegerField(default=0)
    sessions_cost = models.IntegerField(default=0)# price before discount 
    doctor_discount = models.IntegerField(default=0)
    voucher_discount = models.IntegerField(default=0)
    amount_required = models.IntegerField(default=0,help_text = "ثمن الجلسة العلاجية بعد تطبيق خصم نقابة او نسبة خصم لشركةاو بدون خصم")
    initial_payment = models.IntegerField(default=0,help_text = "المبلغ المدفوع < مقابل   أول زيارة أو توقيت الحجز >")
    related_payment = models.FloatField(default=0,help_text = (""))
    amount_paid = models.IntegerField(default=0,help_text = "المبلغ المدفوع < مقابل الجلسة العلاجية المقدمة>")
    amount_remaining = models.IntegerField(default=0,help_text = " لو اتبقي علي المريض فلوس في الزيارة دي أو للمريض فلوس")
    sessions_count = models.IntegerField(default=0)#لما يغيب هنضيفله جلسه هنا بس الاصلية مش هتتغير 
    sessions_count_original = models.IntegerField(default=0)
    sessions_done = models.IntegerField(default=0)
    sessions_absence = models.IntegerField(default=0)
    sessions_remain = models.IntegerField(default=0)
    sessions_done_cost = models.IntegerField(default=0)
    note = models.CharField(max_length=100,blank=True, null=True)
    is_package = models.BooleanField(default=True)
    created_by = models.ForeignKey(User,on_delete=models.PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    is_initial_wa_sent = models.BooleanField(default=False)
    is_debt_hide = models.BooleanField(default=False)
    def __str__(self):
        return "{}-{}".format(self.patient.fullname,self.session.name)

SESSION_ENTRY_STATUS_CHOICES = (
    ("1I", "عند الدكتور"),#INSIDE
    ("2W", "انتظار"),# WAITING , MSTNI DOORO 
    ("3N", "جديد"),# WAITING , MSTNI DOORO 
    #("3C", "تأخير"),# LATE , DOORO GEH W PATINET MSH MWGOD
    ("4F", "خروج"),#FINISHED VISIT , 5LS W MSHI
    ("5A", "غياب"),# ABSENSE , ELYOUM 3ADDA WHWA MAAGSH 
    # mmkn el case elli t7t di aw mmkn a7zfoo w5las 
    ("6C", "إلغاء"),# Cancel , geh 7agaz wmshi  w 3mltoo t25ir wmgash 5als aw geh w la8a w 5d floso 
)
ASSESSMENT_CHOICES = (
    ("UD", "Undefined"),
    ("B", "Bonus"),
    ("D", "Deduction"),
)
ASSESSMENT_CODE_CHOICES = (
    ("P", "تقييم 1"),#"Primary"
    ("S", "تقييم 2"),#Secondary
)
class PsiAssessment (models.Model):
    #notes = models.CharField(max_length=150,blank=True,null=True)
    assessment_choice = models.CharField(max_length=2, choices=ASSESSMENT_CHOICES , default="UD")
    sessions_count = models.IntegerField(default=0)
    doctor = models.ForeignKey(User,related_name="PsiAssessment_doctor", blank=True, null=True, on_delete=models.PROTECT,)
    patient = models.ForeignKey( Patient, related_name="PsiAssessment_patient", blank=True, null=True, on_delete=models.PROTECT, )
    created_by = models.ForeignKey(User,related_name="PsiAssessment_created_by", blank=True, null=True, on_delete=models.PROTECT,)
    created_at = models.DateTimeField(auto_now_add=True)
    assessment_code = models.CharField(max_length=1, choices=ASSESSMENT_CODE_CHOICES , default="P")
    def __str__(self):
        return f"{self.id}>>{self.assessment_code}"
    
class PatientSessionItem(models.Model):
    session_day = models.DateField(blank=True,null=True)
    hour_slot = models.ForeignKey(HourSlot,blank=True,null=True,on_delete=models.PROTECT,related_name="PatientSessionItem_hour_slot",)#twe2eet el galsa 
    session_status = models.ForeignKey(SessionStatus,blank=True,null=True,on_delete=models.PROTECT) 
    session = models.ForeignKey(
        PatientSession,
        related_name="PatientSessionItem_session",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    #amount_paid = models.IntegerField(default=0,help_text = " المبلغ المدفوع من إجمالي الجلسات الحالية >")
    entry_datetime = models.DateTimeField(blank=True,null=True)
    finish_datetime = models.DateTimeField(blank=True,null=True)
    psi_order = models.IntegerField(default=0)
    doctor = models.ForeignKey(User,on_delete=models.PROTECT,related_name="PSI_doctor",null=True)
    reception_notes = models.CharField(max_length=250,blank=True,null=True)#reception
    #entry_order = models.IntegerField(default=1000,help_text = _(" كل يوم جديد يبدأ من أول رقم 1  , ترتيب الدخول في الفترة الحالية"))
    notes_current = models.TextField(blank=True, null=True)#Doctor Notes for Current Session
    notes_next = models.TextField(blank=True, null=True)#Doctor Notes for Next Session
    entry_status = models.CharField(max_length=2, choices=SESSION_ENTRY_STATUS_CHOICES , default="3N")
    created_by = models.ForeignKey(User,on_delete=models.PROTECT,related_name="PSI_created_by")
    created_at = models.DateTimeField(auto_now_add=True)
    is_confirmed = models.BooleanField(default=False)
    is_wa_confirm_sent = models.BooleanField(default=False)
    confirmed_by = models.ForeignKey(User,on_delete=models.PROTECT,blank=True, null=True,related_name="PSI_confirmed_by")
    confirmed_at = models.DateTimeField(blank=True, null=True)
    is_reminder_sent = models.BooleanField(default=False)
    evaluator_note = models.CharField(max_length=150,blank=True, null=True)
    evaluated_by = models.ForeignKey(User,on_delete=models.PROTECT,blank=True, null=True,related_name="PSI_evaluated_by")
    evaluated_at = models.DateTimeField(blank=True, null=True)
    evaluation_status = models.CharField(max_length=2, choices=SESSION_ENTRY_STATUS_CHOICES , default="3N")
    psi_assessment_choice = models.CharField(max_length=2, choices=ASSESSMENT_CHOICES , default="UD")#primary
    assessment = models.ForeignKey(PsiAssessment,on_delete=models.PROTECT,blank=True, null=True,related_name="psi_assessment")
    psi_assessment_choice_s = models.CharField(max_length=2, choices=ASSESSMENT_CHOICES , default="UD")#secondary
    assessment_s = models.ForeignKey(PsiAssessment,on_delete=models.PROTECT,blank=True, null=True,related_name="psi_assessment_s")
    class Meta:
        unique_together = (('session_day', 'session'))

    def __str__(self):
        return "{}-{}".format(self.session.patient.fullname,self.session.session.name)
    
class PtSsnPayment(models.Model):
    pt_ssn =  models.ForeignKey(
        PatientSession,
        related_name="PtSsnPayment_pt_ssn",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    amount_paid = models.IntegerField(default=0,help_text = (" المبلغ المدفوع < مقابل الخدمة العلاجية المقدمة>"))
    notes = models.CharField(max_length=150,blank=True,null=True)
    created_by = models.ForeignKey(User,on_delete=models.PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    is_wa_sent = models.BooleanField(default=False)
    def __str__(self):
        return self.pt_ssn.patient.fullname
'''
from django.db.models.signals import post_save
from django.dispatch import receiver

@receiver(post_save, sender=PtSsnPayment)
def PtSsnPayment_post_save(sender, instance, created, **kwargs):
    if created and instance.amount_paid > 0:
        pt_ssn = PatientSession.objects.get(pk=instance.pt_ssn.pk)
        total_related_payment = pt_ssn.related_payment + instance.amount_paid
        pt_ssn.related_payment = total_related_payment
        pt_ssn.amount_remaining = pt_ssn.amount_required - ( pt_ssn.amount_paid + total_related_payment)
        pt_ssn.save()
'''
class PtSsnDebt(models.Model):
    pt_ssn =  models.OneToOneField(
        PatientSession,
        related_name="pt_ssn_debt",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    notes = models.CharField(max_length=150,blank=True,null=True)
    updated_by = models.ForeignKey(User,on_delete=models.PROTECT, related_name='PtSsnDebt_updated_by',blank=True, null=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.pt_ssn.patient.fullname
    
